import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingException;

import org.apache.commons.lang.time.DateUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import com.justep.appCommon.daisy.DaisyConfig;
import com.justep.appCommon.daisy.DaisyHelper;
import com.justep.appCommon.daisy.FileInfo;
import com.justep.appCommon.daisy.FileType;
import com.justep.appCommon.daisy.MimeType;
import com.justep.filesystem.FileSystemWrapper;
import com.justep.model.ModelUtils;
import com.justep.system.data.*;
import com.justep.system.opm.api.OrgHelper;
import com.justep.system.opm.api.OrgKind;
import com.justep.system.opm.api.PersonHelper;
import com.justep.system.opm.api.ValidState;
import com.justep.system.util.CommonUtils;

public class X5 {
	private static final String DATA_X3 = "/X3Upgrade/X3/data";
	private static final String DATA_X5_SYS = "/system/data";
	private static final String DATA_X5_DOC = "/SA/doc/data";

	private static final String CONFIG_ORG_UPGRADE = "/X3Upgrade/X5/process/orgUpgrade/orgUpgradeConfig.xml";
	
	private static final String DEFAULT_DOC_ROOT = "root";

	public static void orgUpgrade() throws NamingException, SQLException, DocumentException {
		Connection x3Conn = ModelUtils.getConnection(DATA_X3);
		try {
			// 升级组织（机构、部门、岗位）
			organUpgrade(x3Conn);
			// 升级人员和人员成员
			personUpgrade(x3Conn);
		} finally {
			x3Conn.close();
			x3Conn = null;
		}
	}

	private static void organUpgrade(Connection x3Conn) throws SQLException, DocumentException {
		Statement x3Stmt = x3Conn.createStatement();
		try {
			// 按X3原有组织顺序遍历
			ResultSet x3Organ = x3Stmt.executeQuery("select * from TOrgSystem o left join TDept d on o.fGUID = d.fGUID "
					+ "	where o.fKind in ('.OGN', '.DPT', '.WGP', '.PTM')" + "	order by o.fPath, o.fSequence ");
			while (x3Organ.next()) {
				String id = x3Organ.getString("fGUID");
				String name = x3Organ.getString("fDisplayName");
				String code = x3Organ.getString("fID");
				String x3OrgKind = x3Organ.getString("fKind");
				
				// 判断组织节点类型，X3工作组转为部门
				OrgKind orgKind = null;
				if (x3OrgKind.equalsIgnoreCase(".OGN")) {
					orgKind = OrgKind.ogn;
				} else if (x3OrgKind.equalsIgnoreCase(".DPT") || x3OrgKind.equalsIgnoreCase(".WGP")) {
					orgKind = OrgKind.dpt;
				} else if (x3OrgKind.equalsIgnoreCase(".PTM")) {
					orgKind = OrgKind.pos;
				}

				// 判断是否禁用
				int deleteLevel = x3Organ.getInt("fDeleteLevel");
				ValidState validState = ValidState.enabled;
				if (deleteLevel != 0) {
					validState = ValidState.disabled;
				}
				
				String parentID = x3Organ.getString("fParentGUID");
				String path = x3Organ.getString("fPath");
				// 判断根机构
				if (path.equalsIgnoreCase("\\ROOT\\")) {
					parentID = null;
				}
				
				// 预留X3组织信息，用于业务数据升级
				Map<String, Object> extValues = new HashMap<String, Object>();
				extValues.put("eX3DptID", x3Organ.getString("fDept"));
				extValues.put("eX3PosID", x3Organ.getString("fPosition"));
				extValues.put("eX3PsnID", x3Organ.getString("fPerson"));
				extValues.put("eX3URL", path + code + x3OrgKind);

				// 读取升级扩展配置
				Document config = getOrgUpgradeConfig();
				List<?> relations = config.selectNodes("/orgUpgrade/organRelationMapping/relation");
				for (int i = 0; i < relations.size(); i++) {
					Element relation = (Element) relations.get(i);
					String x5Relation = relation.attributeValue("x5");
					String x3Relation = relation.attributeValue("x3");
					extValues.put(x5Relation, x3Organ.getObject(x3Relation));
				}

				// 用组织API添加X5组织节点
				if (!isX5OrgExist(id)) {
					OrgHelper.appendOrg(id, code, name, null, orgKind, validState, parentID, extValues);
					System.out.println("new org->" + x3Organ.getString("fDisplayName"));
				}
			}
		} finally {
			x3Stmt.close();
			x3Stmt = null;
		}
	}

	// 读取组织升级配置文件
	private static Document getOrgUpgradeConfig() throws DocumentException {
		String rootPath = FileSystemWrapper.instance().getBase();
		String configFile = rootPath + CONFIG_ORG_UPGRADE;
		SAXReader reader = new SAXReader();
		Document dom = reader.read(configFile);
		return dom;
	}

	// 人员和人员成员升级
	private static void personUpgrade(Connection x3Conn) throws SQLException, DocumentException {
		Statement x3Stmt = x3Conn.createStatement();
		try {
			// 遍历所有人员成员，注意这里的顺序是优先创建有效的人员成员，否则会产生“人员禁用但人员成员启用”的逻辑错误
			ResultSet x3Person = x3Stmt.executeQuery("select o.*, p.*, p.fGUID as PersonGUID from TOrgSystem o join TPerson p on o.fID = p.fID "
					+ "	where o.fKind in ('.PSM') " + "	order by o.fDeleteLevel, o.fPath, o.fSequence ");
			while (x3Person.next()) {
				String id = x3Person.getString("PersonGUID");
				String name = x3Person.getString("fDisplayName");
				String code = x3Person.getString("fID");
				String idCard = x3Person.getString("eIDCard");
				String x3OrgKind = x3Person.getString("fKind");
				
				// 判断状态
				int deleteLevel = x3Person.getInt("fDeleteLevel");
				ValidState validState = ValidState.enabled;
				if (deleteLevel != 0) {
					validState = ValidState.disabled;
				}
				
				// 判断父
				String parentID = x3Person.getString("fParentGUID");
				String path = x3Person.getString("fPath");
				if (path.equalsIgnoreCase("\\ROOT\\")) {
					parentID = null;
				}
				
				// 预留X3人员信息，用于业务数据升级
				Map<String, Object> extValues = new HashMap<String, Object>();
				extValues.put("eX3PsnID", code);

				// 读取扩展配置
				Document config = getOrgUpgradeConfig();
				List<?> relations = config.selectNodes("/orgUpgrade/personRelationMapping/relation");
				for (int i = 0; i < relations.size(); i++) {
					Element relation = (Element) relations.get(i);
					String x5Relation = relation.attributeValue("x5");
					String x3Relation = relation.attributeValue("x3");
					if (x5Relation.equalsIgnoreCase("sBirthday") || x5Relation.equalsIgnoreCase("sJoinDate")) {
						Date value = x3Person.getDate(x3Relation);
						extValues.put(x5Relation, value);
					} else {
						String value = x3Person.getString(x3Relation);
						extValues.put(x5Relation, value);
					}
				}

				// 构造X5人员成员ID 
				String psmID = id + "@" + parentID;
				if (!isX5PersonExist(id)) {
					// 如果人员不存在，增加X5人员及第一个人员成员
					PersonHelper.appendPerson(id, code, name, null, null, idCard, parentID, extValues);
					if (validState == ValidState.disabled) {
						PersonHelper.disablePerson(id, null);
					}
					System.out.println("new person->" + x3Person.getString("fDisplayName"));
				} else if (!isX5OrgExist(psmID)) {
					// 如果人员成员不存在，增加X5人员成员
					OrgHelper.appendPersonMember(id, parentID, validState, true);
					System.out.println("new personMember->" + x3Person.getString("fDisplayName"));
				}
				
				// 追加组织预留信息
				Map<String, Object> orgExtValues = new HashMap<String, Object>();
				orgExtValues.put("eX3DptID", x3Person.getString("fDept"));
				orgExtValues.put("eX3PosID", x3Person.getString("fPosition"));
				orgExtValues.put("eX3PsnID", x3Person.getString("fPerson"));
				orgExtValues.put("eX3URL", path + code + x3OrgKind);
				OrgHelper.updateOrg(psmID, null, code, name, null, orgExtValues);
			}
		} finally {
			x3Stmt.close();
			x3Stmt = null;
		}
	}

	// 判断X5人员是否存在
	private static boolean isX5PersonExist(String personID) {
		Table table = KSQL.select("select SA_OPPerson from SA_OPPerson SA_OPPerson where SA_OPPerson = '" + personID + "'", null, DATA_X5_SYS, null);
		return table.iterator().hasNext();
	}

	// 判断X5组织是否存在
	private static boolean isX5OrgExist(String orgID) {
		Table table = KSQL.select("select SA_OPOrg from SA_OPOrg SA_OPOrg where SA_OPOrg = '" + orgID + "'", null, DATA_X5_SYS, null);
		return table.iterator().hasNext();
	}

	// 文档目录升级
	public static void docFolderUpgrade() throws NamingException, SQLException {
		Connection x3Conn = ModelUtils.getConnection(DATA_X3);
		try {
			Statement x3Stmt = x3Conn.createStatement();
			try {
				// 遍历X3所有文档目录
				ResultSet rsDoc = x3Stmt.executeQuery("select * from TDocSystem where fIsDirectory = 'Y' order by fPath ");
				try {
					while (rsDoc.next()) {
						String id = rsDoc.getString("fID");
						String name = rsDoc.getString("fDisplayName");
						String path = rsDoc.getString("fPath");
						// 迁移文档目录
						transferDocFolder(id, path, name);
					}
				} finally {
					rsDoc.close();
					rsDoc = null;
				}
			} finally {
				x3Stmt.close();
				x3Stmt = null;
			}
		} finally {
			x3Conn.close();
			x3Conn = null;
		}
	}

	// 迁移文档目录
	private static void transferDocFolder(String id, String x3Path, String name) {
		if ("DOC".equalsIgnoreCase(id)) {
			id = DEFAULT_DOC_ROOT;
		}

		Table x5DocTable = KSQL
				.select("Select SA_DocNode.* From SA_DocNode SA_DocNode where SA_DocNode = '" + id + "'", null, DATA_X5_DOC, null);
		// 判断是否已迁移
		if (x5DocTable.iterator().hasNext()) {
			return;
		}
		
		// 路径转换
		String x5Path = convertDocPath(x3Path);

		// TODO 针对和黄的特殊处理，去掉2级以后的目录
		if (x5Path.split("/").length > 2) {
			return;
		}
		
		String parentID = CommonUtils.getNameNoExtOfFile(x5Path);

		// 生成X5文档目录
		Row row = x5DocTable.appendRow();
		row.setString("SA_DocNode", id);
		row.setString("sParentID", parentID);
		row.setString("sDocName", name);
		row.setString("sDocPath", x5Path);
		row.setString("sKind", "dir");
		row.setInt("sFlag", 1);
		row.setInt("version", 0);
		x5DocTable.save(DATA_X5_DOC);
	}

	// 文档路径转换，所有目录都转换到默认的X5根目录下
	// \878A1666F62026C40665833793D5A234\ -------> /root/878A1666F62026C40665833793D5A234
	private static String convertDocPath(String x3Path) {
		String s = x3Path.replace('\\', '/');
		s = s.replace("/DOC/", "/");
		
		s = "/" + DEFAULT_DOC_ROOT + s;
		
		if (s.length() > 1) {
			s = s.substring(0, s.length() - 1);
		}
		return s;
	}

	// 文档文件升级
	public static int docUpgrade(java.sql.Date beginDate, java.sql.Date endDate, String folder) throws Exception {
		Connection x3Conn = ModelUtils.getConnection(DATA_X3);
		if (endDate != null) {
			endDate = new java.sql.Date(DateUtils.addDays(endDate, 1).getTime());
		}
		
		try {
			// 遍历X3指定日期范围内的文档文件
			PreparedStatement x3Stmt = x3Conn.prepareStatement("select * from TDocSystem " 
					+ "	where (? is null or TDocSystem.fLastWriteTime  >= ?) "
					+ "		and (? is null or TDocSystem.fLastWriteTime < ?) " 
					+ "		and (TDocSystem.fIsDirectory = 'N') "
					+ "	order by TDocSystem.fLastWriteTime ");
			try {
				if (beginDate == null) {
					x3Stmt.setNull(1, java.sql.Types.DATE);
					x3Stmt.setNull(2, java.sql.Types.DATE);
				} else {
					x3Stmt.setDate(1, beginDate);
					x3Stmt.setDate(2, beginDate);
				}
				if (endDate == null) {
					x3Stmt.setNull(3, java.sql.Types.DATE);
					x3Stmt.setNull(4, java.sql.Types.DATE);
				} else {
					x3Stmt.setDate(3, endDate);
					x3Stmt.setDate(4, endDate);
				}
				ResultSet rsDoc = x3Stmt.executeQuery();
				try {
					int errorCount = 0;
					while (rsDoc.next()) {
						String id = rsDoc.getString("fID");
						String fileName = rsDoc.getString("fFileName");
						String displayName = rsDoc.getString("fDisplayName");
						String x3Path = rsDoc.getString("fPath");
						String creatorFID = rsDoc.getString("fCreatorURL");
						String creatorName = rsDoc.getString("fCreatorName");
						Timestamp createTime = rsDoc.getTimestamp("fCreateTime");
						String lastWriterFID = rsDoc.getString("fLastWritorURL");
						String lastWriterName = rsDoc.getString("fLastWritorName");
						Timestamp lastWriteTime = rsDoc.getTimestamp("fLastWriteTime");
						try {
							// 迁移文档文件
							transferDocFile(folder, id, fileName, displayName, x3Path, creatorFID, creatorName, createTime, lastWriterFID, lastWriterName, lastWriteTime);
						} catch (Exception e)  {
							// 控制台输出错误，并记录错误数
							System.out.println(fileName + "(" + displayName + ")导入失败！---------------------------------------------");
							System.out.println(e.getMessage());
							errorCount++;
						}
					}
					return errorCount;
				} finally {
					rsDoc.close();
					rsDoc = null;
				}
			} finally {
				x3Stmt.close();
				x3Stmt = null;
			}
		} finally {
			x3Conn.close();
			x3Conn = null;
		}
	}

	// 迁移文档文件，每个文件都是独立事务提交的
	private static void transferDocFile(String folder, String id, String fileName, String displayName, String x3Path, 
		String creatorFID, String creatorName, Timestamp createTime,
		String lastWriterFID, String lastWriterName, Timestamp lastWriteTime) throws SQLException, IOException, DocumentException, NamingException {
		
		Table x5DocTable = KSQL.select("Select SA_DocNode.* From SA_DocNode SA_DocNode where SA_DocNode = '" + id + "'", null, DATA_X5_DOC, null);
		// 判断是否已经导入
		if (x5DocTable.iterator().hasNext()) {
			return;
		}
		
		// 上传文件
		DaisyConfig config = DaisyConfig.createByDefault(false);
		DaisyHelper helper = new DaisyHelper(config);
		File file = new File(folder, fileName);
		FileInfo fileInfo = null;
		FileInputStream fis = new FileInputStream(file);
		try {
			fileInfo = helper.upload(FileType.attachment, displayName, fis);
		} finally {
			fis.close();
		}
		
		// 文档路径转换
		String x5Path = convertDocPath(x3Path);

		// TODO 针对和黄的特殊处理，去掉2级以后的目录
		String[] ids = x5Path.split("/");
		if (x5Path.split("/").length > 2) {
			x5Path = "/" + ids[1] + "/" + ids[2]; 
		}
		
		String parentID = CommonUtils.getNameNoExtOfFile(x5Path);

		// 保存数据到SA_DocNode
		Connection x5Conn = ModelUtils.getConnection(DATA_X5_DOC);
		try {
			PreparedStatement pstmt = x5Conn.prepareStatement("Insert Into SA_DocNode (sID, sParentID, sDocName, sSize, sKind, sDocPath, sCreatorFID, sCreatorName, sCreateTime, sLastWriterFID, sLastWriterName, sLastWriteTime, sFileID, sDocLiveVersionID, version, sFlag) " +
					"	Values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, 0, 1) ");
			pstmt.setString(1, id);
			pstmt.setString(2, parentID);
			pstmt.setString(3, displayName);
			pstmt.setFloat(4, Float.valueOf(getFileSize(file)));
			pstmt.setString(5, MimeType.getFileMime(fileName));
			pstmt.setString(6, x5Path);
			pstmt.setString(7, creatorFID);
			pstmt.setString(8, creatorName);
			pstmt.setTimestamp(9, createTime);
			pstmt.setString(10, lastWriterFID);
			pstmt.setString(11, lastWriterName);
			pstmt.setTimestamp(12, lastWriteTime);
			pstmt.setString(13, fileInfo.getId());
			pstmt.setInt(14, Integer.parseInt(fileInfo.getLastVersionId()));
			pstmt.execute();
		} finally {
			x5Conn.close();
			x5Conn = null;
		}
	}
	
	private static int getFileSize(File file) throws IOException {
		if (!file.exists()) {
			return 0;
		} else {
			FileInputStream fs = new FileInputStream(file);
			try {
				return fs.available();
			} finally {
				fs.close();
			}
		}
	}

}